<?php

/**
 * @file
 * Contains \Drupal\mason\MasonAdmin.
 */

namespace Drupal\mason;

use Drupal\Core\Url;
use Drupal\Core\Render\Element;
use Drupal\Component\Utility\Html;
use Drupal\Core\Cache\Cache;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\mason\MasonManagerInterface;

/**
 * Provides resusable admin functions or form elements.
 */
class MasonAdmin implements MasonAdminInterface {

  /**
   * The mason manager service.
   *
   * @var \Drupal\mason\MasonManagerInterface.
   */
  protected $manager;

  /**
   * Constructs a MasonAdmin object.
   *
   * @param \Drupal\mason\MasonManagerInterface $manager
   *   The mason manager service.
   */
  public function __construct(MasonManagerInterface $manager) {
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static ($container->get('mason.manager'));
  }

  /**
   * Returns shared form elements across Mason field formatter and Views.
   */
  public function openingForm(array &$form, $definition = []) {
    $path    = drupal_get_path('module', 'mason');
    $readme  = Url::fromUri('base:' . $path . '/README.txt')->toString();
    $vanilla = '';
    if (isset($definition['vanilla'])) {
      $vanilla = ' form--vanilla';
    }
    $form['opening'] = [
      '#markup' => '<div class="form--mason form--slick form--half has-tooltip' . $vanilla . '">',
      '#weight' => -110,
    ];

    $form['vanilla'] = [
      '#type'        => 'checkbox',
      '#title'       => t('Vanilla mason'),
      '#description' => t('<strong>Check</strong>:<ul><li>To render mason item as is as a vanilla Mason.</li><li>To disable 99% Mason features, and most of the mentioned options here, such as layouts, et al.</li><li>Things may be broken! You are on your own.</li></ul><strong>Uncheck</strong>:<ul><li>To get consistent Mason box markups and its advanced features -- relevant for the provided options as Mason needs to know what to style/work with.</li></ul>'),
      '#weight'      => -109,
      '#enforced'    => TRUE,
      '#access'      => isset($definition['vanilla']),
      '#wrapper_attributes' => ['class' => ['form-item--full', 'form-item--tooltip-bottom']],
    ];

    $form['optionset'] = [
      '#type'        => 'select',
      '#title'       => t('Optionset main'),
      '#options'     => $this->getOptionsetOptions(),
      '#enforced'    => TRUE,
      '#description' => t('Enable mason UI module to manage the optionsets.'),
    ];

    if ($this->manager->getModuleHandler()->moduleExists('mason_ui')) {
      $form['optionset']['#description'] = t('Manage optionsets at <a href=":url" target="_blank">Mason admin page</a>.', [':url' => Url::fromRoute('entity.mason.collection')->toString()]);
    }

    $form['skin'] = [
      '#type'        => 'select',
      '#title'       => t('Skin main'),
      '#options'     => $this->getSkinOptions(),
      '#enforced'    => TRUE,
      '#description' => t('Skins allow various layouts with just CSS. Some options below depend on a skin. However a combination of skins and options may lead to unpredictable layouts, get yourself dirty. See <a href=":url" target="_blank">SKINS section at README.txt</a> for details on Skins. Leave empty to DIY. Or use hook_mason_skins_info() and implement \Drupal\mason\MasonSkinInterface to register ones.', [':url' => $readme]),
    ];

    $form['background'] = [
      '#type'        => 'checkbox',
      '#title'       => t('Use CSS background'),
      '#description' => t('If trouble with image sizes not filling the given mason box, check this to turn the image into CSS background instead.'),
    ];

    $lazyload_options = [];
    if ($this->manager->getModuleHandler()->moduleExists('blazy')) {
      $lazyload_options['b-lazy'] = t('bLazy');
    }

    // @todo no lazyload modules for D8 as of this writing, 2/5/2016
    // if ($this->manager->getModuleHandler()->moduleExists('lazyloader')) {
    // $lazyload_options['lazyloader'] = t('Lazyloader');
    // }

    $form['lazyload'] = [
      '#type'        => 'select',
      '#title'       => t('Lazy load'),
      '#options'     => $lazyload_options,
      '#description' => t('Currently only supports blazy.module. Leave empty to DIY.'),
    ];

    $form['layout'] = [
      '#type'        => 'select',
      '#title'       => t('Layout'),
      '#options'     => isset($definition['layouts']) ? $this->getLayoutOptions() + $definition['layouts'] : $this->getLayoutOptions(),
      '#description' => t('Requires a skin. The builtin layouts affects the entire boxes uniformly. Leave empty to DIY.'),
      '#weight'      => 2,
    ];

    $form['caption'] = [
      '#type'        => 'checkboxes',
      '#title'       => t('Caption fields'),
      '#options'     => isset($definition['captions']) ? $definition['captions'] : [],
      '#description' => t('Enable any of the following fields as box caption. These fields are treated and wrapped as captions.'),
      '#access'      => isset($definition['captions']),
      '#weight'       => 80,
    ];

    $weight = -99;
    foreach (Element::children($form) as $key) {
      if (!isset($form[$key]['#weight'])) {
        $form[$key]['#weight'] = ++$weight;
      }
    }
  }

  /**
   * Returns re-usable mason main form.
   */
  public function mainForm(array &$form, $definition = []) {
    $is_colorbox   = function_exists('colorbox_theme');
    $is_photobox   = function_exists('photobox_theme');
    $is_responsive = function_exists('responsive_image_get_image_dimensions');
    $image_styles  = function_exists('image_style_options') ? image_style_options(FALSE) : [];
    $photobox      = \Drupal::root() . '/libraries/photobox/photobox/jquery.photobox.js';

    if (is_file($photobox)) {
      $is_photobox = TRUE;
    }

    $form['media_switch'] = [
      '#type'        => 'select',
      '#title'       => t('Media switcher'),
      '#options'     => [
        'content' => t('Image linked to content'),
      ],
      '#description' => t('Depends on the enabled supported modules, or has known integration with Mason.<ol><li>Link to content: for aggregated small masons.</li><li>Image to iframe: audio/video is hidden below image until toggled, otherwise iframe is always displayed, and draggable fails. Aspect ratio applies.</li><li>Colorbox.</li><li>Photobox. Be sure to select "Thumbnail style" for the overlay thumbnails.</li><li>Intense: image to fullscreen intense image.</li></ol>'),
      '#prefix' => '<h3 class="form--slick__title">' . t('Fields') . '</h3>',
      '#access'      => isset($definition['switchers']),
    ];

    // Optional lightbox integration.
    if (isset($definition['switchers']) && ($is_colorbox || $is_photobox)) {
      if ($is_colorbox) {
        $form['media_switch']['#options']['colorbox'] = t('Image to colorbox');
      }

      if ($is_photobox) {
        $form['media_switch']['#options']['photobox'] = t('Image to photobox');
      }

      // Re-use the same image style for both lightboxes.
      $form['box_style'] = [
        '#type'    => 'select',
        '#title'   => t('Lightbox image style'),
        '#options' => $image_styles,
        '#states'  => [
          'visible' => [
            'select[name*="[media_switch]"]' => [['value' => 'colorbox'], ['value' => 'photobox']],
          ],
        ],
      ];
    }

    $form['image'] = [
      '#type'        => 'select',
      '#title'       => t('Main image'),
      '#options'     => isset($definition['images']) ? $definition['images'] : [],
      '#description' => t('Main background/stage image field. Only expects an image formatter by now.'),
      '#access'      => isset($definition['images']),
    ];

    $form['title'] = [
      '#type'        => 'select',
      '#title'       => t('Title'),
      '#options'     => isset($definition['titles']) ? $definition['titles'] : [],
      '#description' => t('If provided, it will be wrapped with H2 and class .box__title.'),
      '#access'      => isset($definition['titles']),
    ];

    $form['link'] = [
      '#type'        => 'select',
      '#title'       => t('Link'),
      '#options'     => isset($definition['links']) ? $definition['links'] : [],
      '#description' => t('Link to content: Read more, View Case Study, etc, wrapped with class .box__link.'),
      '#access'      => isset($definition['links']),
    ];

    $form['fillers'] = [
      '#type'        => 'select',
      '#title'       => t('Filler start at'),
      '#options'     => array_combine(range(1, 42), range(1, 42)),
      '#description' => t('Index to mark contents as fillers. Contents starting from this value will be treated as fillers. Be sure the total Views rows are bigger than this. Mason uses fillers to fill in gaps. Fillers are elements that you can define or it will reuse elements within the grid. Leave it empty to disable fillers, and use Promoted option instead to control sizes.'),
    ];

    $form['class'] = [
      '#type'        => 'select',
      '#title'       => t('Box class'),
      '#options'     => isset($definition['classes']) ? $definition['classes'] : [],
      '#description' => t('If provided, individual box will have this class. Be sure its formatter is Key.'),
      '#access'      => isset($definition['classes']),
      '#weight'      => 6,
    ];

    $form['id'] = [
      '#type'         => 'textfield',
      '#title'        => t('Mason ID'),
      '#size'         => 40,
      '#maxlength'    => 255,
      '#field_prefix' => '#',
      '#enforced'     => TRUE,
      '#description'  => t("Manually define the Mason container ID. <em>This ID is used for the cache identifier, so be sure it is unique</em>. Leave empty to have a guaranteed unique ID managed by the module."),
      '#access'       => isset($definition['id']),
      '#weight'       => 94,
    ];
  }

  /**
   * Returns shared ending form elements across Mason field formatter and Views.
   */
  public function closingForm(array &$form, $definition = []) {
    $form['cache'] = [
      '#type'        => 'select',
      '#title'       => t('Cache'),
      '#options'     => $this->getCacheOptions(),
      '#weight'      => 98,
      '#enforced'    => TRUE,
      '#description' => t('Ditch all the mason logic to cached bare HTML. <ol><li><strong>Permanent</strong>: cached contents will persist (be displayed) till the next cron runs.</li><li><strong>Any number</strong>: expired by the selected expiration time, and fresh contents are fetched till the next cache rebuilt.</li></ol>A working cron job is required to clear stale cache. At any rate, cached contents will be refreshed regardless of the expiration time after the cron hits. <br />Leave it empty to disable caching.<br /><strong>Warning!</strong> Be sure no useless/ sensitive data such as Edit links as they are rendered as is regardless permissions. Only enable it when all is done, otherwise cached options will be displayed while changing them.'),
    ];

    $form['current_view_mode'] = [
      '#type'          => 'hidden',
      '#default_value' => isset($definition['current_view_mode']) ? $definition['current_view_mode'] : '_custom',
      '#weight'        => 100,
    ];

    $form['closing'] = [
      '#markup' => '</div>',
      '#weight' => 110,
    ];

    if ($this->manager->getModuleHandler()->moduleExists('slick')) {
      $admin = \Drupal::service('slick.admin');
      $admin->finalizeForm($form, $definition);
    }
    else {
      $this->finalizeForm($form, $definition);
    }
    $form['#attached']['library'][] = 'mason/mason.admin';
  }

  /**
   * Returns re-usable logic, styling and assets across Mason fields and Views.
   */
  public function finalizeForm(array &$form, $definition = []) {
    $settings = isset($definition['settings']) ? $definition['settings'] : [];

    foreach (Element::children($form) as $key) {
      if (isset($form[$key]['#type']) && !in_array($form[$key]['#type'], ['container', 'details', 'item', 'hidden'])) {
        if (!isset($form[$key]['#default_value']) && isset($settings[$key])) {
          $form[$key]['#default_value'] = $settings[$key];
        }
        if (!isset($form[$key]['#attributes']) && isset($form[$key]['#description'])) {
          $form[$key]['#attributes'] = ['class' => ['is-tooltip']];
        }

        if ($form[$key]['#type'] == 'select' && !in_array($key, ['cache', 'optionset', 'view_mode'])) {
          if (!isset($form[$key]['#empty_option']) && !isset($form[$key]['#required'])) {
            $form[$key]['#empty_option'] = t('- None -');
          }
        }

        if (!isset($form[$key]['#enforced']) && isset($definition['vanilla'])) {
          $states['visible'][':input[name*="[vanilla]"]'] = ['checked' => FALSE];
          if (isset($form[$key]['#states'])) {
            $form[$key]['#states']['visible'][':input[name*="[vanilla]"]'] = ['checked' => FALSE];
          }
          else {
            $form[$key]['#states'] = $states;
          }
        }
      }
    }
  }

  /**
   * Returns time in interval for select options.
   */
  public function getCacheOptions() {
    $period = [0, 60, 180, 300, 600, 900, 1800, 2700, 3600, 10800, 21600, 32400, 43200, 86400];
    $period = array_map([\Drupal::service('date.formatter'), 'formatInterval'], array_combine($period, $period));
    $period[0] = '<' . t('No caching') . '>';
    return $period + [Cache::PERMANENT => t('Permanent')];
  }

  /**
   * Returns available mason optionsets for select options.
   */
  public function getOptionsetOptions() {
    $optionsets = [];
    $masons = $this->manager->loadMultiple('mason');
    foreach ((array) $masons as $mason) {
      $optionsets[$mason->id()] = Html::escape($mason->label());
    }
    asort($optionsets);
    return $optionsets;
  }

  /**
   * Returns available mason skins for select options.
   */
  public function getSkinOptions() {
    $skins = &drupal_static(__METHOD__, NULL);
    if (!isset($skins)) {
      $skins = [];
      foreach ($this->manager->getSkins() as $skin => $properties) {
        $skins[$skin] = Html::escape($properties['name']);
      }
    }

    return $skins;
  }

  /**
   * Returns default layout options for the core Image, or Views.
   */
  public function getLayoutOptions() {
    return [
      'bottom' => t('Caption bottom'),
      'top'    => t('Caption top'),
      'center' => t('Caption center'),
    ];
  }

}
