<?php

/**
 * @file
 * Contains \Drupal\mason\MasonManager.
 */

namespace Drupal\mason;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Component\Utility\NestedArray;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\mason\Entity\Mason;

/**
 * Implements MasonManagerInterface.
 *
 * @todo lazyload, granular image styles, decent skins, stamps.
 */
class MasonManager implements MasonManagerInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface;
   */
  protected $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Constructs a MasonManager object
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, RendererInterface $renderer, CacheBackendInterface $cache) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler     = $module_handler;
    $this->renderer          = $renderer;
    $this->cache             = $cache;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('renderer'),
      $container->get('cache.default')
    );
  }

  /**
   * Returns the entity type manager.
   */
  public function getEntityTypeManager() {
    return $this->entityTypeManager;
  }

  /**
   * Returns the module handler.
   */
  public function getModuleHandler() {
    return $this->moduleHandler;
  }

  /**
   * Returns the renderer.
   */
  public function getRenderer() {
    return $this->renderer;
  }

  /**
   * Returns a shortcut for loading a configuration entity.
   */
  public function load($id, $entity_type = 'mason') {
    return $this->entityTypeManager->getStorage($entity_type)->load($id);
  }

  /**
   * Returns a shortcut for loading multiple configuration entities.
   */
  public function loadMultiple($entity_type = 'mason', $ids = NULL) {
    return $this->entityTypeManager->getStorage($entity_type)->loadMultiple($ids);
  }

  /**
   * Returns available mason default options.
   */
  public function getDefaultOptions() {
    return $this->load('default')->getOptions();
  }

  /**
   * Returns available mason default options.
   */
  public function getDefaultOption($option_name) {
    return $this->load('default')->getOption($option_name);
  }

  /**
   * Returns defined mason skins as registered via hook_mason_skins_info().
   */
  public function getSkins() {
    $skins = &drupal_static(__METHOD__, NULL);
    if (!isset($skins)) {
      $cid = 'mason:skins';
      if ($cache = $this->cache->get($cid)) {
        $skins = $cache->data;
      }
      else {
        $classes = $this->moduleHandler->invokeAll('mason_skins_info');
        $classes = array_merge(['\Drupal\mason\MasonSkin'], $classes);
        $items   = $skins = [];
        foreach ($classes as $class) {
          if (class_exists($class)) {
            $reflection = new \ReflectionClass($class);
            if ($reflection->implementsInterface('\Drupal\mason\MasonSkinInterface')) {
              $skin = new $class;
              $items = $skin->skins();
            }
          }
          $skins = NestedArray::mergeDeep($skins, $items);
        }
        $tags = Cache::buildTags($cid, ['count:' . count($items)]);
        $this->cache->set($cid, $skins, Cache::PERMANENT, $tags);
      }
    }
    return $skins;
  }

  /**
   * Returns array of needed assets suitable for #attached for the given mason.
   */
  public function attach($attach = []) {
    $attach += ['skin' => FALSE];

    if (!empty($attach['lazyload']) && $attach['lazyload'] == 'b-lazy') {
      $load['library'][] = 'blazy/blazy';
    }

    $load['library'][] = 'mason/mason';
    $load['library'][] = 'mason/mason.load';

    $skins = $this->getSkins();
    if ($skin = $attach['skin']) {
      $provider = isset($skins[$skin]['provider']) ? $skins[$skin]['provider'] : 'mason';
      $load['library'][] = 'mason/' . $provider . '.' . $skin;
    }

    $load['drupalSettings']['mason'] = $this->getDefaultOptions();

    $this->moduleHandler->alter('mason_attach_info', $load, $attach);
    return $load;
  }

  /**
   * {@inheritdoc}
   */
  public function build($build = []) {
    foreach (['items', 'optionset', 'settings'] as $key) {
      $build[$key] = isset($build[$key]) ? $build[$key] : [];
    }

    $mason = [
      '#theme'      => 'mason',
      '#items'      => [],
      '#build'      => $build,
      '#pre_render' => [[$this, 'preRenderMason']],
    ];

    $settings          = $build['settings'];
    $suffixes[]        = count($build['items']);
    $suffixes[]        = count(array_filter($settings));
    $suffixes[]        = $settings['cache'];
    $cache['tags']     = Cache::buildTags('mason:' . $settings['id'], $suffixes, '.');
    $cache['contexts'] = ['languages'];
    $cache['max-age']  = $settings['cache'];
    $cache['keys']     = isset($settings['cache_metadata']['keys']) ? $settings['cache_metadata']['keys'] : [$settings['id']];
    $mason['#cache']   = $cache;

    return $mason;
  }

  /**
   * {@inheritdoc}
   */
  public function preRenderMason($element) {
    $build = $element['#build'];
    unset($element['#build']);

    if (empty($build['items'])) {
      return [];
    }

    // Build mason elements.
    $defaults    = Mason::htmlSettings();
    $settings    = $build['settings'] ? array_merge($defaults, $build['settings']) : $defaults;
    $attachments = $this->attach($settings);
    $optionset   = $build['optionset'] ?: $this->load($settings['optionset']);

    $element['#optionset'] = $optionset;
    $element['#settings']  = $settings;
    $element['#attached']  = empty($build['attached']) ? $attachments : NestedArray::mergeDeep($build['attached'], $attachments);
    $element['#items']     = $build['items'];
    $element['#json']      = $optionset->getJson();

    return $element;
  }

  /**
   * Returns basic plugin settings.
   */
  public static function defaultSettings() {
    return [
      'background'        => TRUE,
      'box_style'         => '',
      'cache'             => -1,
      'caption'           => [],
      'class'             => '',
      'current_view_mode' => '',
      'fillers'           => '',
      'image'             => '',
      'layout'            => '',
      'lazyload'          => '',
      'link'              => '',
      'media_switch'      => '',
      'optionset'         => 'default',
      'ratio'             => '',
      'skin'              => '',
      'stamp'             => [],
      'title'             => '',
      'vanilla'           => FALSE,
    ];
  }

}
