<?php

/**
 * @file
 * Contains \Drupal\mason\Plugin\views\style\MasonViews.
 */

namespace Drupal\mason\Plugin\views\style;

use Drupal\Component\Utility\Xss;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\views\Plugin\views\style\StylePluginBase;
use Drupal\mason\Entity\Mason;
use Drupal\mason\MasonManagerInterface;

/**
 * Mason style plugin.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "mason",
 *   title = @Translation("Mason"),
 *   help = @Translation("Display the results in a Mason."),
 *   theme = "mason",
 *   register_theme = FALSE,
 *   display_types = {"normal"}
 * )
 */
class MasonViews extends StylePluginBase {

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesGrouping = FALSE;

  /**
   * Constructs a MasonManager object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MasonManagerInterface $manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('mason.manager'));
  }

  /**
   * Returns the mason admin.
   */
  public function admin() {
    return \Drupal::service('mason.admin');
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = ['id' => ['default' => '']];
    foreach ($this->manager->defaultSettings() as $key => $value) {
      $options[$key] = ['default' => $value];
    }
    return $options + parent::defineOptions();
  }

  /**
   * Overrides \Drupal\views\Plugin\views\style\StylePluginBase::buildOptionsForm().
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $admin       = $this->admin();
    $field_names = $this->displayHandler->getFieldLabels();
    $definitions = [
      'vanilla',
      'captions',
      'layouts',
      'images',
      'links',
      'titles',
      'classes',
      'id',
    ];

    $definition = [];
    foreach ($this->displayHandler->getOption('fields') as $field => $handler) {
      if (isset($handler['type'])) {
        switch ($handler['type']) {
          case 'image':
          // case 'media':
            $definition['images'][$field] = $field_names[$field];
            break;

          case 'list_key':
            $definition['layouts'][$field] = $field_names[$field];
            break;

          case 'entity_reference':
          case 'text':
          case 'string':
          case 'link':
            $definition['links'][$field] = $field_names[$field];
            $definition['titles'][$field] = $field_names[$field];
            break;
        }

        if (in_array($handler['type'], ['list_key', 'entity_reference', 'text', 'string'])) {
          $definition['classes'][$field] = $field_names[$field];
        }
      }

      // Content: title is not really a field, unless title.module installed.
      if (isset($handler['field'])) {
        if ($handler['field'] == 'title') {
          $definition['classes'][$field] = $field_names[$field];
          $definition['titles'][$field] = $field_names[$field];
        }

        if ($handler['field'] == 'view_node') {
          $definition['links'][$field] = $field_names[$field];
        }
      }

      // Captions can be anything to get custom works going.
      $definition['captions'][$field] = $field_names[$field];
    }

    $definition['settings'] = $this->options;
    $definition['current_view_mode'] = $this->view->current_display;
    foreach ($definitions as $key) {
      $definition[$key] = isset($definition[$key]) ? $definition[$key] : [];
    }

    $admin->openingForm($form, $definition);
    $admin->mainForm($form, $definition);
    $admin->closingForm($form, $definition);

    $title = '<p class="form--slick__header form--slick__title">';
    $title .= $this->t('Check Vanilla mason for custom markups.<small>Otherwise mason markups are added. Add the supported fields to appear here.</small>');
    $title .= '</p>';
    $form['opening']['#markup'] = '<div class="form--mason form--slick form--views form--half form--vanilla has-tooltip">' . $title;
  }

  /**
   * Overrides StylePluginBase::render().
   */
  public function render() {
    $view      = $this->view;
    $settings  = $this->options;
    $view_name = $view->storage->id();
    $view_mode = $view->current_display;
    $count     = count($view->result);
    $id        = Mason::getHtmlId("mason-views-{$view_name}-{$view_mode}", $settings['id']);

    $settings += [
      'count'             => $count,
      'current_view_mode' => $view_mode,
      'caption'           => array_filter($settings['caption']),
      'view_name'         => $view_name,
      'id'                => $id,
      'cache_metadata'    => ['keys'=> [$id, $view_mode, $settings['optionset']]],
    ];

    $elements = [];
    foreach ($this->renderGrouping($view->result, $settings['grouping']) as $rows) {
      $build = [
        'items'    => $this->buildElements($rows, $settings),
        'settings' => $settings,
      ];

      $elements = $this->manager->build($build);
    }
    return $elements;
  }

  /**
   * Returns mason contents.
   */
  public function buildElements($rows, $settings = []) {
    $build = [];
    $view  = $this->view;
    $keys  = array_keys($view->field);

    foreach ($rows as $index => $row) {
      $view->row_index = $index;

      $box['box'] = $box['image'] = '';
      $box['settings'] = $settings;

      if (!empty($settings['class'])) {
        $classes = $this->getFieldString($row, $settings['class'], $index);
        $box['settings']['class'] = empty($classes[$index]) ? [] : $classes[$index];
      }

      // Use Vanilla mason if so configured, ignoring Mason markups.
      if ($settings['vanilla']) {
        $box['box'] = $view->rowPlugin->render($row);
      }
      else {
        // Add main image fields if so configured.
        if ($field_image = $settings['image']) {
          $box['box']   = $settings['background'] ? '' : $this->getFieldRendered($index, $field_image);
          $box['image'] = $this->extractImageData($row, $index, $field_image);
        }

        // Add caption fields if so configured.
        $box['caption']['title'] = $this->getFieldRendered($index, $settings['title'], TRUE);
        $box['caption']['link']  = $this->getFieldRendered($index, $settings['link']);

        // Add extra caption fields if so configured.
        if ($captions = $settings['caption']) {
          $caption_items = [];
          foreach ($captions as $key => $caption) {
            $caption_rendered = $this->getField($index, $caption);
            if (empty($caption_rendered)) {
              continue;
            }

            if (in_array($caption, array_values($keys))) {
              $caption_items[$key]['#markup'] = $caption_rendered;
            }
          }
          $box['caption']['data'] = $caption_items;
        }

        // Add layout field, may be a list field, or builtin layout options.
        if ($layout = $settings['layout']) {
          if (strpos($layout, 'field_') !== FALSE) {
            $settings['layout'] = strip_tags($this->getField($index, $layout));
          }
          $box['settings']['layout'] = $settings['layout'];
        }
      }

      // Build mason items.
      $build[] = $box;
    }

    unset($view->row_index);
    return $build;
  }

  /**
   * Returns the image data suitable for CSS background image, and extra works.
   */
  public function extractImageData($row, $index, $field_image) {
    $image = $this->getFieldRenderable($row, $index, $field_image);
    if (!isset($image['rendered']['#item'])) {
      return [];
    }
    $item        = $image['rendered']['#item'];
    $file        = $item->getEntity()->get($field_image);
    $uri         = $file->referencedEntities()[0]->getFileUri();
    $image_style = $image['rendered']['#image_style'];
    $image_url   = $item->entity->url();

    if (!empty($image_style)) {
      $style = $this->manager->load($image_style, 'image_style');
      $image_url = $style->buildUrl($uri);
    }

    return [
      'target_id'   => $item->getValue()['target_id'],
      'uri'         => $uri,
      'image_url'   => $image_url,
      'image_style' => $image_style,

      // @todo lightboxes based on media switcher.
      'url'         => $image['rendered']['#url'],
    ];
  }

  /**
   * Returns the rendered field, either string or array.
   *
   * @todo consider a small re-usable module to avoid Slick duplication.
   */
  public function getFieldRendered($index, $field_name = '', $restricted = FALSE) {
    if (!empty($field_name) && $output = $this->getField($index, $field_name)) {
      return is_array($output) ? $output : ['#markup' => ($restricted ? Xss::filterAdmin($output) : $output)];
    }
    return [];
  }

  /**
   * Returns the renderable array of field containing rendered and raw data.
   */
  public function getFieldRenderable($row, $index, $field_name = '', $multiple = FALSE) {
    if (empty($field_name)) {
      return;
    }
    $result = $this->view->field[$field_name]->getItems($row);
    return $multiple ? $result : $result[0];
  }

  /**
   * Returns the string values for the expected Title, ET, List, Term.
   */
  public function getFieldString($row, $field_name, $idx) {
    $values   = [];
    $renderer = $this->manager->getRenderer();

    // Content title/List/Text, either as link or plain text.
    if ($value = $this->getFieldString($idx, $field_name)) {
      $value = is_string($value) ? $value : (isset($value[0]['value'])? $value[0]['value'] : '');
      $values[$idx] = empty($value) ? '' : Html::cleanCssIdentifier(Unicode::strtolower($value));
    }

    // Term reference/ET, either as link or plain text.
    if ($renderable = $this->getFieldRenderable($row, $field_name, TRUE)) {
      $value = [];
      foreach ($renderable as $key => $render) {
        $class = isset($render['rendered']['#title']) ? $render['rendered']['#title'] : $renderer->render($render['rendered']);
        $class = strip_tags($class);
        $value[$key] = Html::cleanCssIdentifier(Unicode::strtolower($class));
      }
      $values[$idx] = empty($value) ? '' : implode(' ', $value);
    }
    return $values;
  }

}
